"""rpdb.py

    A remote debugger for Python 2.3.3

    This is a modification of pdb.py to allow remoting of the debugger IO to a
    seperate server process, and support convenient debugging of multiple
    threads.

    New debugger commands:
        ts (Thread Set) - Set debugger focus to a specific thread, or list
            active threads.
        tc (Thread Command) - Send command to a specific thread.
        bn (Break Now) - Break all threads immediately, or as soon as they
            return to execute python code.
        save - Save breakpoints of the current debug session.
        load - Load breakpoints of the current debug session.

    Other improvements:
        Post mortem on unhandled exceptions.
        Debugger starts in right frame.

    To debug a thread (except the main thread):
        1. Place the line 'rpdb.set_trace()' as the first line in the run()
        method of your thread. This line will be ignored if the script was not
        invoked by the debugger, so it can remain in the code.
        2. When execution reaches that line the debugger will break and the
        debugger-server will report the new thread.

        Important note: when a new thread starts under the debugger, it is
        initially stopped and has to be continued manually, but the rest of
        the debugee's threads may be running!

        The name of the thread in focus of the debugger-server will appear
        at the prompt.
        The alias '*' stands for all debugged threads. Any command issued
        when '*' appears at the prompt will be sent to all debugged threads!

        If you want to debug a script that is run from an embedded interpreter,
        add 'rpdb.set_active()' in the beginning of your script. This will
        make the calls to 'rpdb.set_trace()' break into the debugger (they
        will not be ignored). Don't forget to add an extra 'rpdb.set_trace()'
        for your main thread. You also need to manually start rpdb.py in
        server mode before you run your script.

    Pdb Limitations:
        1. Unlike C++ debuggers you can't always break into a running thread.
        A thread may be doing system code or call non Pythonish code. The
        debugger will stop the thread as soon as it returns to python code.
        2. When one thread is stopped by the debugger other threads may be
        running.
        3. When executing a general Python statement from the debugger-server
        console, any IO of that statement will be done in the debugee console.

    Other issues:
        On Linux, shutting down the server with the close-window button, may
        leave it running in the background. In this case you should either
        kill the server process manually or start the next debug session with
        a differnt port.

    This is a "beta" version. I will appreciate any feedback!

    Remoting code and modifications written by Ziaran:
        n i r 1 4 0 8 @ h o t m a i l . c o m
    """

# (See pdb.doc for documentation.)



import sys
import linecache
import cmd
import bdb
from repr import Repr
import os
import re
import pprint

#
# imports needed for the remote engine
#
import threading
import socket
import getopt
import string
import time
import pickle



# Create a custom safe Repr instance and increase its maxstring.
# The default of 30 truncates error messages too easily.
_repr = Repr()
_repr.maxstring = 200
_saferepr = _repr.repr

mainmodule = ''
mainpyfile = ''

__all__ = ["run", "pm", "Pdb", "runeval", "runctx", "runcall", "set_trace",
           "set_active", "post_mortem", "help"]

#
# Globals for the remote engine
#
PDBRC_FILENAME = '.pdbrc'
CONSOLE_RESPONSE_TIME = 2
OUTPUT_AQUIRE_TIME_SEC = 0.1

CONTROL_CHANNEL = 'control'
BREAKPOINTS_FILE_EXTENSION = 'bpl'
ALIAS_PREFIX = 'Thread-'
MAIN_THREAD_NAME = 'MainThread'

DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 50009

g_host = DEFAULT_HOST
g_port = DEFAULT_PORT

g_fActive = False

#
# map between OS type and relvant command to initiate a new OS console.
# entries for other OSs can be added here.
# '%s' serves as a place holder.
#
osSpawn = {'nt': 'start cmd /c python.exe %s', 'posix': 'xterm -e python %s &'}



def FindFile(filename, FirstPath = ''):
    """Expand filname to explicit path."""
    if os.path.isfile(os.path.join(FirstPath, filename)):
        return filename

    path = os.environ['PATH']
    paths = [''] + sys.path + string.split(path, os.pathsep)

    for p in paths:
        f = os.path.join(p, filename)
        if os.path.isfile(f):
            return f



class CAutoLock:
    def __init__(self, lock):
        self.m_lock = lock
        self.m_lock.acquire()

    def __del__(self):
        self.m_lock.release()



class COutputSync:
    """Output sync on server side for multiple clients.
    This class prevents multiple-lines response from threads from being
    interleaved. It also lets the console know when all responses arrived."""
    def __init__(self):
        self.m_lock = threading.RLock()
        self.m_owner = None
        self.m_clock = 0.0
        self.m_fIdle = True

    def Aquire(self, duration):
        """Aquire ownership for duration (seconds).
        Return True if ownership was won over."""
        while True:
            lock = CAutoLock(self.m_lock)
            contender = str(threading.currentThread())
            clock = time.time()
            if self.m_owner == contender:
                self.m_clock = clock + duration
                self.m_fIdle = False
                return False

            if clock >= self.m_clock:
                self.m_clock = clock + duration
                self.m_owner = contender
                fOwnershipWon = not self.m_fIdle
                self.m_fIdle = False
                return fOwnershipWon

            del lock
            sleep = (self.m_clock - clock) * 1.1 + 0.001
            time.sleep(sleep)

    def WaitUntilIdle(self):
        """Wait until all aquisition activity ceases.
        Not guranteed!"""
        if self.m_clock == 0:
            self.m_fIdle = True
            return

        while True:
            clock = time.time()
            if clock >= self.m_clock:
                self.m_fIdle = True
                return
            sleep = (self.m_clock - clock) * 1.1 + 0.1
            time.sleep(sleep)



class CConsole(cmd.Cmd):
    def __init__(self, server):
        cmd.Cmd.__init__(self)
        self.m_server = server
        self.m_currentAlias = '*'
        self.prompt = self.m_currentAlias + ' > '

    def postcmd(self, stop, line):
        if sum(self.m_server.GetExpectedResponse().values()) == 0:
            return stop

        #
        # Wait until connections respond to command.
        #
        fIsEventSet = self.m_server.WaitResponseEvent(CONSOLE_RESPONSE_TIME)

        if not fIsEventSet:
            unresponsive = self.m_server.GetExpectedResponse()
            for alias in unresponsive.keys():
                if unresponsive[alias] == 1:
                    print 'Connection', alias, 'is not responsive.It may be running, ' \
                    'blocked on system call, or doing non Pythonish code.'
            unresponsive.clear()

        #
        # Wait untill all output activity ceases.
        #
        self.m_server.WaitUntilIdleOutput()
        print ''
        return stop

    def do_save(self, arg):
        self.m_server.WriteAlias(CONTROL_CHANNEL, 'save')

    def do_load(self, arg):
        self.m_server.WriteAlias(CONTROL_CHANNEL, 'load')

    def do_bn(self, arg):
        self.m_server.WriteAlias(CONTROL_CHANNEL, 'break')

    def do_ts(self, arg):
        """Set focus to particular connection or list all connections."""
        if not arg:
            self.m_server.Report()
            return

        if arg == CONTROL_CHANNEL:
            print 'You can not set focus to that thread.'
            return

        alias = '*'

        if arg != '*':
            client, alias = self.m_server.GetClient(arg)
            if not client:
                print 'No such thread.'
                return

        #
        # '*' means all connections will be in focus.
        #
        self.m_currentAlias = alias
        self.prompt = self.m_currentAlias + ' > '

    def do_tc(self, arg):
        """Send command to particular connection."""
        args = string.split(arg)
        if len(args) == 0:
            print 'Arguments excpected.\nType "help tc" for more info.'
            return

        command = string.join(args[1:])
        alias = args[0]
        if alias == CONTROL_CHANNEL:
            print 'You can not send to that connection.'
            return

        self.m_server.WriteAlias(alias, command)

    def do_quit(self, arg):
        self.m_server.WriteAll('quit')
        self.m_server.WriteAlias(CONTROL_CHANNEL, 'exit')
        return True

    do_q = do_quit

    def do_jump(self, arg):
        if self.m_currentAlias == '*' and arg:
            print "jump can not be issued from global focus ('*'). Set " \
                "focus to a responsive thread (with the 'ts' command) and " \
                "then try to jump again."
            return

        self.m_server.WriteAlias(self.m_currentAlias, 'jump ' + arg)

    do_j = do_jump

    def do_break(self, arg, temporary = 0):
        if self.m_currentAlias == '*' and arg:
            print "Breakpoints can not be set from global focus ('*') to " \
                "avoid unexpected behavior (see help for more info). Set " \
                "focus to a responsive thread (with the 'ts' command) and " \
                "then try to set the breakpoint again. All threads will see " \
                "that breakpoint and break on it."
            return

        self.m_server.WriteAlias(self.m_currentAlias, (temporary and 't' or '') + 'break ' + arg)

    do_b = do_break

    def do_tbreak(self, arg):
        self.do_break(arg, 1)

    def default(self, line):
        """Forward command to connection in focus."""
        self.m_server.WriteAlias(self.m_currentAlias, line)

    def do_help(self, arg):
        """Override Cmd.do_help() since do_* methods are on the clients. Only
        the help_* were copied here."""
        if arg:
            try:
                func = getattr(self, 'help_' + arg)
                func()
            except AttributeError:
                self.stdout.write("%s\n"%str(self.nohelp % (arg,)))
            return

        names = self.get_names()
        names.sort()
        cmds_doc = []
        for name in names:
            if name[:5] == 'help_':
                cmds_doc.append(name[5:])

        self.stdout.write("%s\n"%str(self.doc_leader))
        self.print_topics(self.doc_header, cmds_doc, 15,80)

    do_h = do_help

    def help_save(self):
        print """save - Save breakpoints to file."""

    def help_load(self):
        print """load - Load breakpoints from file."""

    def help_bn(self):
        print """bn - (Break Now)
When issued, will cause each of the debugged threads to break immediately or
once they become responsive."""

    def help_quit(self):
        print """q(uit) or exit - Quit from the debugger.
The program being executed is aborted."""

    help_q = help_quit

    def help_ts(self):
        print """ts [thread-name | '*']
With an argument, set debugger focus to the thread of that name.
Without an argument, list names of all active threads.
'ts N' and 'ts m' can be used as shorthand notation for 'ts Thread-N' and
'ts MainThread' respectivly.
'ts *' will set focus back to all threads."""

    def help_tc(self):
        print """tc (thrad-name | '*') [cmd]
Send cmd as command to the thread of named thread-name.
'tc N ...' and 'tc m ...' can be used as shorthand notation for
'tc Thread-N ...' and 'tc MainThread ...' respectivly.
'tc * ...' sends cmd as command to all active threads."""

    # Help methods (derived from pdb.doc)

    def help_help(self):
        self.help_h()

    def help_h(self):
        print """h(elp)
Without argument, print the list of available commands.
With a command name as argument, print help about that command
"help pdb" pipes the full documentation file to the $PAGER
"help exec" gives help on the ! command"""

    def help_where(self):
        self.help_w()

    def help_w(self):
        print """w(here)
Print a stack trace, with the most recent frame at the bottom.
An arrow indicates the "current frame", which determines the
context of most commands.  'bt' is an alias for this command."""

    help_bt = help_w

    def help_down(self):
        self.help_d()

    def help_d(self):
        print """d(own)
Move the current frame one level down in the stack trace
(to an older frame)."""

    def help_up(self):
        self.help_u()

    def help_u(self):
        print """u(p)
Move the current frame one level up in the stack trace
(to a newer frame)."""

    def help_break(self):
        self.help_b()

    def help_b(self):
        print """b(reak) ([file:]lineno | function) [, condition]
With a line number argument, set a break there in the current
file.  With a function name, set a break at first executable line
of that function.  Without argument, list all breaks.  If a second
argument is present, it is a string specifying an expression
which must evaluate to true before the breakpoint is honored.

The line number may be prefixed with a filename and a colon,
to specify a breakpoint in another file (probably one that
hasn't been loaded yet).  The file is searched for on sys.path;
the .py suffix may be omitted."""

    def help_clear(self):
        self.help_cl()

    def help_cl(self):
        print "cl(ear) filename:lineno"
        print """cl(ear) [bpnumber [bpnumber...]]
With a space separated list of breakpoint numbers, clear
those breakpoints.  Without argument, clear all breaks (but
first ask confirmation).  With a filename:lineno argument,
clear all breaks at that line in that file.

Note that the argument is different from previous versions of
the debugger (in python distributions 1.5.1 and before) where
a linenumber was used instead of either filename:lineno or
breakpoint numbers."""

    def help_tbreak(self):
        print """tbreak  same arguments as break, but breakpoint is
removed when first hit."""

    def help_enable(self):
        print """enable bpnumber [bpnumber ...]
Enables the breakpoints given as a space separated list of
bp numbers."""

    def help_disable(self):
        print """disable bpnumber [bpnumber ...]
Disables the breakpoints given as a space separated list of
bp numbers."""

    def help_ignore(self):
        print """ignore bpnumber count
Sets the ignore count for the given breakpoint number.  A breakpoint
becomes active when the ignore count is zero.  When non-zero, the
count is decremented each time the breakpoint is reached and the
breakpoint is not disabled and any associated condition evaluates
to true."""

    def help_condition(self):
        print """condition bpnumber str_condition
str_condition is a string specifying an expression which
must evaluate to true before the breakpoint is honored.
If str_condition is absent, any existing condition is removed;
i.e., the breakpoint is made unconditional."""

    def help_step(self):
        self.help_s()

    def help_s(self):
        print """s(tep)
Execute the current line, stop at the first possible occasion
(either in a function that is called or in the current function)."""

    def help_next(self):
        self.help_n()

    def help_n(self):
        print """n(ext)
Continue execution until the next line in the current function
is reached or it returns."""

    def help_return(self):
        self.help_r()

    def help_r(self):
        print """r(eturn)
Continue execution until the current function returns."""

    def help_continue(self):
        self.help_c()

    def help_cont(self):
        self.help_c()

    def help_c(self):
        print """c(ont(inue))
Continue execution, only stop when a breakpoint is encountered."""

    def help_jump(self):
        self.help_j()

    def help_j(self):
        print """j(ump) lineno
Set the next line that will be executed."""

    def help_debug(self):
        print """debug code
Enter a recursive debugger that steps through the code argument
(which is an arbitrary expression or statement to be executed
in the current environment)."""

    def help_list(self):
        self.help_l()

    def help_l(self):
        print """l(ist) [first [,last]]
List source code for the current file.
Without arguments, list 11 lines around the current line
or continue the previous listing.
With one argument, list 11 lines starting at that line.
With two arguments, list the given range;
if the second argument is less than the first, it is a count."""

    def help_args(self):
        self.help_a()

    def help_a(self):
        print """a(rgs)
Print the arguments of the current function."""

    def help_p(self):
        print """p expression
Print the value of the expression."""

    def help_pp(self):
        print """pp expression
Pretty-print the value of the expression."""

    def help_exec(self):
        print """(!) statement
Execute the (one-line) statement in the context of
the current stack frame.
The exclamation point can be omitted unless the first word
of the statement resembles a debugger command.
To assign to a global variable you must always prefix the
command with a 'global' command, e.g.:
(Pdb) global list_options; list_options = ['-l']
(Pdb)"""

    def help_quit(self):
        self.help_q()

    def help_q(self):
        print """q(uit) or exit - Quit from the debugger.
The program being executed is aborted."""

    help_exit = help_q

    def help_whatis(self):
        print """whatis arg
Prints the type of the argument."""

    def help_EOF(self):
        print """EOF
Handles the receipt of EOF as a command."""

    def help_alias(self):
        print """alias [name [command [parameter parameter ...] ]]
Creates an alias called 'name' the executes 'command'.  The command
must *not* be enclosed in quotes.  Replaceable parameters are
indicated by %1, %2, and so on, while %* is replaced by all the
parameters.  If no command is given, the current alias for name
is shown. If no name is given, all aliases are listed.

Aliases may be nested and can contain anything that can be
legally typed at the pdb prompt.  Note!  You *can* override
internal pdb commands with aliases!  Those internal commands
are then hidden until the alias is removed.  Aliasing is recursively
applied to the first word of the command line; all other words
in the line are left alone.

Some useful aliases (especially when placed in the .pdbrc file) are:

#Print instance variables (usage "pi classInst")
alias pi for k in %1.__dict__.keys(): print "%1.",k,"=",%1.__dict__[k]

#Print instance variables in self
alias ps pi self
"""

    def help_unalias(self):
        print """unalias name
Deletes the specified alias."""

    def help_pdb(self):
        help()



class CDescriptorBridge(threading.Thread):
    """Bridge an input file descriptor to output file descriptor."""
    def __init__(self, alias, server, fdin, fdout):
        threading.Thread.__init__(self)
        assert fdin and fdout
        self.setDaemon(True)
        self.m_server = server
        self.m_fdin = fdin
        self.m_fdout = fdout
        self.m_stop = False
        self.m_alias = alias
        self.m_prompt = alias + ' : '
        self.m_buffer = ''

    def Stop(self):
        self.m_stop = True

    def run(self):
        while not self.m_stop:
            try:
                chr = self.m_fdin.read(1)
                if not chr:
                    break
            except socket.error, args:
                print >> sys.stderr, socket.error, args
                break

            self.m_buffer += chr
            if chr == '\n':
                #
                # Aquire output ownership, since other bridges may be waiting
                # to write to this output file.
                #
                fNeedNewline = self.m_server.AquireOutput(self.m_alias, OUTPUT_AQUIRE_TIME_SEC)
                if fNeedNewline:
                    self.m_fdout.write('\n')
                self.m_fdout.write(self.m_prompt + self.m_buffer)
                self.m_buffer = ''



class CIOServer(threading.Thread):
    """Serve and manage incoming clients (pdb connections)."""
    def __init__(self, port):
        threading.Thread.__init__(self)
        self.setDaemon(True)
        self.m_port = port
        self.m_socket = None
        self.m_mClients = {}
        self.m_stop = False
        self.m_OutputTimeStamp = 0
        self.m_OutputSync = COutputSync()
        self.m_mExpectedResponse = {}
        self.m_eResponse = threading.Event()

    def Stop(self):
        self.m_stop = True

    def run(self):
        self.m_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.m_socket.bind(('', self.m_port))

        print 'Started server on port', self.m_port

        while not self.m_stop:
            self.m_socket.listen(1)
            conn, addr = self.m_socket.accept()

            try:
                fdin = conn.makefile('r')
                alias = fdin.readline()
                alias = alias[:-1]
            except socket.error:
                continue

            #
            # Bridge this connection to stdout
            #
            bridge = CDescriptorBridge(alias, self, fdin, sys.stdout)
            bridge.start()

            self.m_mClients[alias] = (conn, bridge)

            print 'Accepted connection from addr:', addr
            print 'Connection alias is', alias

    def WaitUntilIdleOutput(self):
        self.m_OutputSync.WaitUntilIdle()

    def AquireOutput(self, alias, duration):
        """Aquire stdout ownership for connection response."""
        fUnExpectedResponse = alias not in self.m_mExpectedResponse
        fOwnershipTaken = self.m_OutputSync.Aquire(duration)
        fNeedNewline = fOwnershipTaken or fUnExpectedResponse
        self.MarkExpectedResponse(alias)
        return fNeedNewline

    def AddToExpectedResponse(self, alias):
        self.m_eResponse.clear()
        self.m_mExpectedResponse[alias] = 1

    def MarkExpectedResponse(self, alias):
        self.m_mExpectedResponse[alias] = 0
        if sum(self.m_mExpectedResponse.values()) == 0:
            #
            # Signal that all expected responses arrived.
            #
            self.m_eResponse.set()

    def GetExpectedResponse(self):
        """Return map of clients that did not yet respond."""
        return self.m_mExpectedResponse

    def WaitResponseEvent(self, duration):
        """Wait 'duration' until all clients respond."""
        self.m_eResponse.wait(duration)
        return self.m_eResponse.isSet()

    def WriteAll(self, str):
        """Send 'str' to all clients."""
        #
        # Don't use iterator since map may change while in loop.
        #
        for alias in [a for a in self.m_mClients.keys() if a != CONTROL_CHANNEL]:
            client = self.m_mClients[alias]
            if not client[1].isAlive():
                #
                # Cleanup dead clients.
                #
                del self.m_mClients[alias]
                self.MarkExpectedResponse(alias)
                continue
            try:
                client[0].send(str + '\n')
                self.AddToExpectedResponse(alias)
            except socket.error, args:
                print >> sys.stderr, socket.error, args
                client[1].Stop()
                del self.m_mClients[alias]
                self.MarkExpectedResponse(alias)

    def GetClient(self, alias):
        try:
            if alias not in self.m_mClients:
                if ALIAS_PREFIX + alias in self.m_mClients:
                    alias = ALIAS_PREFIX + alias
                elif alias == 'm' and MAIN_THREAD_NAME in self.m_mClients:
                    alias = MAIN_THREAD_NAME
                else: return None, None

            client = self.m_mClients[alias]
            return client, alias
        except KeyError:
            print >> sys.stderr, alias, 'is not connected.'

        return None, None

    def WriteAlias(self, alias, str):
        """Send 'str' to client connection with alias 'alias'."""
        if alias == '*':
            self.WriteAll(str)
            return
        try:
            client, alias = self.GetClient(alias)
            if not client:
                return
            client[0].send(str + '\n')
            self.AddToExpectedResponse(alias)
        except socket.error, args:
            print >> sys.stderr, socket.error, args
            client[1].Stop()
            del self.m_mClients[alias]
            self.MarkExpectedResponse(alias)

    def Report(self):
        """List all active connections."""
        print 'Active threads:'
        for alias in [a for a in self.m_mClients.keys() if a != CONTROL_CHANNEL]:
            print alias



class CIOClient:
    """Setup client connection and cleanup on destruction."""
    def __init__(self, name, host, port):
        self.m_name = name
        self.m_host = host
        self.m_port = port
        self.m_socket = None
        self.m_fdin = None
        self.m_fdout = None

    def connect(self):
        self.m_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        while True:
            #
            # Loop until connection established (server is setup).
            #
            try:
                self.m_socket.connect((self.m_host, self.m_port))
                self.m_socket.send(self.m_name + '\n')
                break
            except socket.error, args:
                print >> sys.stderr, 'socket.error', args
                time.sleep(1)

        self.m_fdin = self.m_socket.makefile('r')
        self.m_fdout = self.m_socket.makefile('w', 0)

#        print >> sys.stderr, 'Connected to server at host', self.m_host, 'on port', self.m_port

    def Flush(self):
        try:
            self.m_socket.setblocking(0)
        except socket.error, instance:
            pass

        try:
            data = self.m_socket.recv(1024)
        except:
            pass

        try:
            self.m_socket.setblocking(1)
        except socket.error, instance:
            pass

    def GetFileIn(self):
        return self.m_fdin

    def GetFileOut(self):
        return self.m_fdout



class CControlChannel(cmd.Cmd, CIOClient, threading.Thread):
    def __init__(self, filename, host, port):
        cmd.Cmd.__init__(self)
        CIOClient.__init__(self, CONTROL_CHANNEL, host, port)
        threading.Thread.__init__(self)
        self.use_rawinput = 0
        self.prompt = ''
        self.setDaemon(True)
        self.m_filename = filename[:filename.rfind('.') + 1] + BREAKPOINTS_FILE_EXTENSION

    def run(self):
        self.connect()
        self.stdin = self.GetFileIn()
        self.stdout = self.GetFileOut()

        try:
            self.cmdloop()
        except socket.error, instance:
            print 'socket.error', instance

    def do_save(self, arg):
        try:
            file = open(self.m_filename, 'w')
        except IOError, instance:
            print >> self.stdout, 'IOError', instance
            return

        object = (Pdb.breaks, bdb.Breakpoint.next, bdb.Breakpoint.bplist, bdb.Breakpoint.bpbynumber)
        pickle.dump(object, file)
        file.close()

        print >> self.stdout, 'Breakpoints saved to', self.m_filename

    def do_load(self, arg):
        try:
            file = open(self.m_filename, 'r')
        except IOError, instance:
            print >> self.stdout, 'IOError', instance
            return

        object = pickle.load(file)
        (Pdb.breaks, bdb.Breakpoint.next, bdb.Breakpoint.bplist, bdb.Breakpoint.bpbynumber) = object
        file.close()

        print >> self.stdout, 'Breakpoints loaded from', self.m_filename

    def do_break(self, arg):
        Pdb.m_nbreakNowIssued += 1
        print >> self.stdout, 'Break now issued.'

    def do_exit(self, arg):
        return True



def StartServer(port):
    server = CIOServer(port)
    server.start()

    time.sleep(2)

    console = CConsole(server)
    console.cmdloop()



def SpawnServer(port):
    """Start an OS console to act as server."""
    assert os.name in osSpawn

    command = osSpawn[os.name] % ('"' + sys.argv[0] + '" --server --port=' + str(port))
    print >> sys.stderr, 'Spawning server with:', command

    os.popen2(command)



def StartClient(host, port, args):
    global mainmodule
    global mainpyfile

    global g_host
    global g_port
    global g_fActive

    g_host = host
    g_port = port
    g_fActive = True

    ExpandedFilename = args[0]

    mainpyfile = ExpandedFilename
    mainmodule = os.path.basename(ExpandedFilename)

    #
    # Insert script directory in front of module search path
    #
    sys.path.insert(0, os.path.dirname(ExpandedFilename))
    sys.argv = args

    controlChannel = CControlChannel(ExpandedFilename, host, port)
    controlChannel.start()

    time.sleep(1)

    try:
        run('execfile(' + `ExpandedFilename` + ')')
    except socket.error, (errno, description):
        print >> sys.stderr, `(errno, description)`
        print >> sys.stderr, 'Server stopped. Terminating debug session.'



def PrintUsage(fExtended = False):
    scriptName = os.path.basename(sys.argv[0])
    print scriptName + """ [-h] [-c [-aN] | -s] [-pM] script-name [script-args...]

    Where the options above mean the following:
    -h, --help     print extended help
    -c, --client   start client side only (server will not spawn automatically)
    -s, --server   start server side only (will wait for clients to connect)
    -aN, --host=N  connect to server on host N
    -pM, --port=M  connect to port M

    Examples:
        rpdb.py my_script.py "hello world"

        Will run my_script.py with arg "hello world" and spawn a server
        console where the debugger IO will be redirected.

    ts, tc, bn, save, and load, are new debugger commands for handling threads
    and breakpoints.
    """

    if not fExtended:
        return

    print __doc__



def main():
    try:
        options, args = getopt.getopt(
                            sys.argv[1:],
                            'hcsa:p:',
                            ['help', 'client', 'server', 'host=', 'port=']
                            )
    except getopt.GetoptError:
        PrintUsage()
        return 2

    host = DEFAULT_HOST
    port = DEFAULT_PORT
    fServerOnly = False
    fClientOnly = False

    for o, a in options:
        if o in ['-h', '--help']:
            PrintUsage(fExtended = True)
            return 0
        if o in ['-a', '--host']:
            host = a
        if o in ['-p', '--port']:
            port = int(a)
        if o in ['-s', '--server']:
            fServerOnly = True
        if o in ['-c', '--client']:
            fClientOnly = True

    if (len(args) == 0 and not fServerOnly) or \
        (fServerOnly and len(args)) or \
        (fServerOnly and host != DEFAULT_HOST):
        PrintUsage()
        return 2

    if not fServerOnly and (os.path.basename(args[0]) == os.path.basename(sys.argv[0])):
        print 'Cannot debug self.'
        return 0

    if fServerOnly:
        StartServer(port)
        return 0

    ExpandedFilename = FindFile(args[0])
    if not ExpandedFilename:
        print 'File', args[0], ' not found.'
        return

    args[0] = ExpandedFilename

    if not fClientOnly:
        SpawnServer(port)

    time.sleep(1)

    StartClient(host, port, args)
    return 0



def find_function(funcname, filename):
    cre = re.compile(r'def\s+%s\s*[(]' % funcname)
    try:
        fp = open(filename)
    except IOError:
        return None
    # consumer of this info expects the first line to be 1
    lineno = 1
    answer = None
    while 1:
        line = fp.readline()
        if line == '':
            break
        if cre.match(line):
            answer = funcname, filename, lineno
            break
        lineno = lineno + 1
    fp.close()
    return answer


# Interaction prompt line will separate file and call info from code
# text using value of line_prefix string.  A newline and arrow may
# be to your liking.  You can set it once pdb is imported using the
# command "pdb.line_prefix = '\n% '".
# line_prefix = ': '    # Use this to get the old situation back
line_prefix = '\n-> '   # Probably a better default

class Pdb(bdb.Bdb, cmd.Cmd):

    #
    # breaks initialization moved from Bdb.__init__() to class to maintain
    # coherent state with Breakpoint.bplist which is also a class member.
    # Needed for multi threading breakpoints to work correctly.
    # This is also why Bdb.__init__ is never invoked.
    #
    breaks = {}
    m_nbreakNowIssued = 0

    def __init__(self, stdin = None, stdout = None, fPostMortem = False):
        cmd.Cmd.__init__(self, 'tab')
        self.fncache = {}
        self.use_rawinput = 0
        self.prompt = ''
        self.aliases = {}

        self.m_ThreadStartFrame = None
        self.m_fPostMortem = fPostMortem

        clientName = threading.currentThread().getName()
        self.m_client = CIOClient(clientName, g_host, g_port)

        if not stdin:
            self.m_client.connect()
            self.stdin = self.m_client.GetFileIn()
            self.stdout = self.m_client.GetFileOut()
        else:
            self.stdin = stdin
            self.stdout = stdout

        self.m_nBreakNowDone = Pdb.m_nbreakNowIssued
        self.fRunInvoked = False

        # Try to load readline if it exists
        try:
            import readline
        except ImportError:
            pass

        # Read $HOME/.pdbrc and ./.pdbrc
        self.rcLines = []

        envHome = ''
        if 'HOME' in os.environ:
            envHome = os.environ['HOME']

        f = FindFile(PDBRC_FILENAME, envHome)
        if f:
            try:
                rcFile = open(f)
            except IOError:
                pass
            else:
                for line in rcFile.readlines():
                    self.rcLines.append(line)
                rcFile.close()
                print >> self.stdout, "Loaded commands from '" + f + "'."

    #
    # Override of Bdb methods to support features like Break Now.
    #

    def trace_dispatch(self, frame, event, arg):
        try:
            return bdb.Bdb.trace_dispatch(self, frame, event, arg)
        except socket.error:
            if self.m_ThreadStartFrame:
                #
                # for threads other than the main thread translate
                # BdbQuit to SystemExit to avoid threading.py
                # dumping exception stack on thread termination.
                #
                raise SystemExit
            else:
                raise
        except bdb.BdbQuit:
            if self.m_ThreadStartFrame:
                #
                # for threads other than the main thread translate
                # BdbQuit to SystemExit to avoid threading.py
                # dumping exception stack on thread termination.
                #
                raise SystemExit
            else:
                raise

    def stop_here(self, frame):
        if self.fRunInvoked:
            #
            # Allow an automatic-extra step after Pdb.run() invoked.
            # This will cause us to start at debugee's frame.
            #
            self.fRunInvoked = False
            return False

        #
        # Break now is handled here
        #
        if Pdb.m_nbreakNowIssued > self.m_nBreakNowDone:
            return True

        return bdb.Bdb.stop_here(self, frame)

    def dispatch_call(self, frame, arg):
        # XXX 'arg' is no longer used
        if self.botframe is None:
            # First call of dispatch since reset()
            self.botframe = frame.f_back # (CT) Note that this may also be None!
            return self.trace_dispatch
        if self.stop_here(frame) or self.break_anywhere(frame):
            self.user_call(frame, arg)
            if self.quitting: raise bdb.BdbQuit
        return self.trace_dispatch

    def dispatch_exception(self, frame, arg):
        #
        # Allow postmortem when non-main thread hits bottom frame with
        # unhandled exception.
        #
        if (frame is self.m_ThreadStartFrame) or self.stop_here(frame):
            self.user_exception(frame, arg)
            if self.quitting: raise bdb.BdbQuit
        return self.trace_dispatch

    def set_continue(self):
        #
        # Don't stop except at breakpoints or when finished
        #
        self.stopframe = self.botframe
        self.returnframe = None
        self.quitting = 0

    def set_trace(self):
        """Start debugging from here."""
        self.reset()
        frame = sys._getframe().f_back
        #
        # Go two more frames back so non-main thread will begin at first line
        # after debugger invocation
        #
        assert frame
        frame = frame.f_back
        assert frame
        self.m_ThreadStartFrame = frame
        while frame:
            frame.f_trace = self.trace_dispatch
            self.botframe = frame
            frame = frame.f_back
        self.set_step()
        sys.settrace(self.trace_dispatch)

    def reset(self):
        bdb.Bdb.reset(self)
        self.forget()

    def forget(self):
        self.lineno = None
        self.stack = []
        self.curindex = 0
        self.curframe = None

    def setup(self, f, t):
        self.forget()
        self.stack, self.curindex = self.get_stack(f, t)
        self.curframe = self.stack[self.curindex][0]
        self.execRcLines()

    # Can be executed earlier than 'setup' if desired
    def execRcLines(self):
        if self.rcLines:
            # Make local copy because of recursion
            rcLines = self.rcLines
            # executed only once
            self.rcLines = []
            for line in rcLines:
                line = line[:-1]
                if len(line) > 0 and line[0] != '#':
                    self.onecmd(line)

    # Override Bdb methods

    def user_call(self, frame, argument_list):
        """This method is called when there is the remote possibility
        that we ever need to stop in this function."""
        if not self.stop_here(frame):
            return

        if Pdb.m_nbreakNowIssued > self.m_nBreakNowDone:
            self.m_nBreakNowDone += 1
            self.m_client.Flush()
            print >> self.stdout, '--Break Now--'

        print >> self.stdout, '--Call--'
        self.interaction(frame, None)

    def user_line(self, frame):
        """This function is called when we stop or break at this line."""
        if Pdb.m_nbreakNowIssued > self.m_nBreakNowDone:
            self.m_nBreakNowDone += 1
            self.m_client.Flush()
            print >> self.stdout, '--Break Now--'

        self.interaction(frame, None)

    def user_return(self, frame, return_value):
        """This function is called when a return trap is set here."""
        if Pdb.m_nbreakNowIssued > self.m_nBreakNowDone:
            self.m_nBreakNowDone += 1
            self.m_client.Flush()
            print >> self.stdout, '--Break Now--'

        frame.f_locals['__return__'] = return_value
        print >> self.stdout, '--Return--'
        self.interaction(frame, None)

    def user_exception(self, frame, (exc_type, exc_value, exc_traceback)):
        """This function is called if an exception occurs,
        but only if we are to stop at or just below this level."""
        if Pdb.m_nbreakNowIssued > self.m_nBreakNowDone:
            self.m_nBreakNowDone += 1
            self.m_client.Flush()
            print >> self.stdout, '--Break Now--'

        if frame is self.m_ThreadStartFrame:
            #
            # Post mortem when non-main thread hits bottom frame with unhandled
            # exception
            #
            print >> self.stdout, exc_type, exc_value
            print >> self.stdout, "*** Unhandled exception."
            print >> self.stdout, "*** Execution terminated."
            print >> self.stdout, "*** Starting post mortem. (You can leave post mortem with the 'EOF' command)"
            post_mortem(exc_traceback, self.stdin, self.stdout)
            self.quitting = 1
            return

        frame.f_locals['__exception__'] = exc_type, exc_value
        if type(exc_type) == type(''):
            exc_type_name = exc_type
        else: exc_type_name = exc_type.__name__
        print >> self.stdout, exc_type_name + ':', _saferepr(exc_value)
        self.interaction(frame, exc_traceback)

    # General interaction function

    def interaction(self, frame, traceback):
        self.setup(frame, traceback)
        self.print_stack_entry(self.stack[self.curindex])
        self.cmdloop()
        self.forget()

    def default(self, line):
        print >> self.stdout, "Statement '" + line + "' executed in the " + \
            "context of the current stack frame. IO will be done in " + \
            "debugee console."

        if line[:1] == '!': line = line[1:]
        locals = self.curframe.f_locals
        globals = self.curframe.f_globals

        try:
            code = compile(line + '\n', '<stdin>', 'single')
            exec code in globals, locals
        except:
            t, v = sys.exc_info()[:2]
            if type(t) == type(''):
                exc_type_name = t
            else: exc_type_name = t.__name__
            print >> self.stdout, '***', exc_type_name + ':', v

    def run(self, cmd, globals=None, locals=None):
        self.fRunInvoked = True
        try:
            bdb.Bdb.run(self, cmd, globals, locals)
        except:
            type, value, tb = sys.exc_info()
            print >> self.stdout, type, value
            print >> self.stdout, "*** Unhandled exception."
            print >> self.stdout, "*** Execution terminated."
            print >> self.stdout, "*** Starting post mortem. (You can leave post mortem with the 'EOF' command)"
            post_mortem(tb, self.stdin, self.stdout)

    def precmd(self, line):
        """Handle alias expansion and ';;' separator."""
        if not line.strip():
            return line
        args = line.split()

        if args[0] not in ['l', 'list']:
            self.lineno = None

        while args[0] in self.aliases:
            line = self.aliases[args[0]]
            ii = 1
            for tmpArg in args[1:]:
                line = line.replace("%" + str(ii),
                                      tmpArg)
                ii = ii + 1
            line = line.replace("%*", ' '.join(args[1:]))
            args = line.split()
        # split into ';;' separated commands
        # unless it's an alias command
        if args[0] != 'alias':
            marker = line.find(';;')
            if marker >= 0:
                # queue up everything after marker
                next = line[marker+2:].lstrip()
                self.cmdqueue.append(next)
                line = line[:marker].rstrip()
        return line

    def postcmd(self, stop, line):
        #
        # Ignore Break-Nows issued while in interactive mode.
        #
        self.m_nBreakNowDone = Pdb.m_nbreakNowIssued
        return stop

    # Command definitions, called by cmdloop()
    # The argument is the remaining string on the command line
    # Return true to exit from the command loop

    do_h = cmd.Cmd.do_help

    def bpprint(self, bp):
        """Copy of bdb.Breakpoint.bpprint that does not break on non stdout
        file descriptors."""
        if bp.temporary:
            disp = 'del  '
        else:
            disp = 'keep '
        if bp.enabled:
            disp = disp + 'yes'
        else:
            disp = disp + 'no '
        print >> self.stdout, '%-4dbreakpoint    %s at %s:%d' % (bp.number, disp,
                             bp.file, bp.line)
        if bp.cond:
            print >> self.stdout, '\tstop only if %s' % (bp.cond,)
        if bp.ignore:
            print >> self.stdout, '\tignore next %d hits' % (bp.ignore)
        if (bp.hits):
            if (bp.hits > 1): ss = 's'
            else: ss = ''
            print >> self.stdout, ('\tbreakpoint already hit %d time%s' %
                   (bp.hits, ss))

    def VerifyNotPostMortem(self, command):
        #
        # Don't allow a command that is not valid for post mortem mode.
        #
        if self.m_fPostMortem:
            print >> self.stdout, "Not possible to issue '" + command + "' in post mortem mode."
            return False
        return True

    def do_break(self, arg, temporary = 0):
        if not self.VerifyNotPostMortem('break'): return

        # break [ ([filename:]lineno | function) [, "condition"] ]
        if not arg:
            if not self.breaks:
                print >> self.stdout, "No breakpoints set."
                return
            print >> self.stdout, "Num Type         Disp Enb   Where"
            for bp in bdb.Breakpoint.bpbynumber:
                if bp:
                    self.bpprint(bp)
            return
        # parse arguments; comma has lowest precedence
        # and cannot occur in filename
        filename = None
        lineno = None
        cond = None
        comma = arg.find(',')
        if comma > 0:
            # parse stuff after comma: "condition"
            cond = arg[comma+1:].lstrip()
            arg = arg[:comma].rstrip()
        # parse stuff before comma: [filename:]lineno | function
        colon = arg.rfind(':')
        if colon >= 0:
            filename = arg[:colon].rstrip()
            f = self.lookupmodule(filename)
            if not f:
                print >> self.stdout, '*** ', `filename`,
                print >> self.stdout, 'not found from sys.path'
                return
            else:
                filename = f
            arg = arg[colon+1:].lstrip()
            try:
                lineno = int(arg)
            except ValueError, msg:
                print >> self.stdout, '*** Bad lineno:', arg
                return
        else:
            # no colon; can be lineno or function
            try:
                lineno = int(arg)
            except ValueError:
                try:
                    func = eval(arg,
                                self.curframe.f_globals,
                                self.curframe.f_locals)
                except:
                    func = arg
                try:
                    if hasattr(func, 'im_func'):
                        func = func.im_func
                    code = func.func_code
                    lineno = code.co_firstlineno
                    filename = code.co_filename
                except:
                    # last thing to try
                    (ok, filename, ln) = self.lineinfo(arg)
                    if not ok:
                        print >> self.stdout, '*** The specified object',
                        print >> self.stdout, `arg`,
                        print >> self.stdout, 'is not a function'
                        print >> self.stdout, ('or was not found '
                               'along sys.path.')
                        return
                    lineno = int(ln)
        if not filename:
            filename = self.defaultFile()
        # Check for reasonable breakpoint
        line = self.checkline(filename, lineno)
        if line:
            # now set the break point
            err = self.set_break(filename, line, temporary, cond)
            if err: print >> self.stdout, '***', err
            else:
                bp = self.get_breaks(filename, line)[-1]
                print >> self.stdout, "Breakpoint %d at %s:%d" % (bp.number,
                                                  bp.file,
                                                  bp.line)

    # To be overridden in derived debuggers
    def defaultFile(self):
        """Produce a reasonable default."""
        filename = self.curframe.f_code.co_filename
        if filename == '<string>' and mainpyfile:
            filename = mainpyfile
        return filename

    do_b = do_break

    def do_tbreak(self, arg):
        self.do_break(arg, 1)

    def lineinfo(self, identifier):
        failed = (None, None, None)
        # Input is identifier, may be in single quotes
        idstring = identifier.split("'")
        if len(idstring) == 1:
            # not in single quotes
            id = idstring[0].strip()
        elif len(idstring) == 3:
            # quoted
            id = idstring[1].strip()
        else:
            return failed
        if id == '': return failed
        parts = id.split('.')
        # Protection for derived debuggers
        if parts[0] == 'self':
            del parts[0]
            if len(parts) == 0:
                return failed
        # Best first guess at file to look at
        fname = self.defaultFile()
        if len(parts) == 1:
            item = parts[0]
        else:
            # More than one part.
            # First is module, second is method/class
            f = self.lookupmodule(parts[0])
            if f:
                fname = f
            item = parts[1]
        answer = find_function(item, fname)
        return answer or failed

    def checkline(self, filename, lineno):
        """Return line number of first line at or after input
        argument such that if the input points to a 'def', the
        returned line number is the first
        non-blank/non-comment line to follow.  If the input
        points to a blank or comment line, return 0.  At end
        of file, also return 0."""

        line = linecache.getline(filename, lineno)
        if not line:
            print >> self.stdout, 'End of file'
            return 0
        line = line.strip()
        # Don't allow setting breakpoint at a blank line
        if (not line or (line[0] == '#') or
             (line[:3] == '"""') or line[:3] == "'''"):
            print >> self.stdout, '*** Blank or comment'
            return 0
        # When a file is read in and a breakpoint is at
        # the 'def' statement, the system stops there at
        # code parse time.  We don't want that, so all breakpoints
        # set at 'def' statements are moved one line onward
        if line[:3] == 'def':
            instr = ''
            brackets = 0
            while 1:
                skipone = 0
                for c in line:
                    if instr:
                        if skipone:
                            skipone = 0
                        elif c == '\\':
                            skipone = 1
                        elif c == instr:
                            instr = ''
                    elif c == '#':
                        break
                    elif c in ('"',"'"):
                        instr = c
                    elif c in ('(','{','['):
                        brackets = brackets + 1
                    elif c in (')','}',']'):
                        brackets = brackets - 1
                lineno = lineno+1
                line = linecache.getline(filename, lineno)
                if not line:
                    print >> self.stdout, 'end of file'
                    return 0
                line = line.strip()
                if not line: continue   # Blank line
                if brackets <= 0 and line[0] not in ('#','"',"'"):
                    break
        return lineno

    def do_enable(self, arg):
        if not self.VerifyNotPostMortem('enable'): return

        args = arg.split()
        for i in args:
            try:
                i = int(i)
            except ValueError:
                print >> self.stdout, 'Breakpoint index %r is not a number' % i
                continue

            if not (1 <= i < len(bdb.Breakpoint.bpbynumber)):
                print >> self.stdout, 'No breakpoint numbered', i
                continue

            bp = bdb.Breakpoint.bpbynumber[i]
            if bp:
                bp.enable()
                print >> self.stdout, 'Enabled breakpoint number', i

    def do_disable(self, arg):
        if not self.VerifyNotPostMortem('disable'): return

        args = arg.split()
        for i in args:
            try:
                i = int(i)
            except ValueError:
                print >> self.stdout, 'Breakpoint index %r is not a number' % i
                continue

            if not (1 <= i < len(bdb.Breakpoint.bpbynumber)):
                print >> self.stdout, 'No breakpoint numbered', i
                continue

            bp = bdb.Breakpoint.bpbynumber[i]
            if bp:
                bp.disable()
                print >> self.stdout, 'Disabled breakpoint number', i

    def do_condition(self, arg):
        if not self.VerifyNotPostMortem('condition'): return

        # arg is breakpoint number and condition
        args = arg.split(' ', 1)
        bpnum = int(args[0].strip())
        if not (1 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
            print >> self.stdout, 'No breakpoint numbered', bpnum
            return

        try:
            cond = args[1]
        except:
            cond = None
        bp = bdb.Breakpoint.bpbynumber[bpnum]
        if bp:
            bp.cond = cond
            if not cond:
                print >> self.stdout, 'Breakpoint', bpnum,
                print >> self.stdout, 'is now unconditional.'

    def do_ignore(self,arg):
        if not self.VerifyNotPostMortem('ignore'): return

        """arg is bp number followed by ignore count."""
        args = arg.split()
        bpnum = int(args[0].strip())
        if not (1 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
            print >> self.stdout, 'No breakpoint numbered', bpnum
            return

        try:
            count = int(args[1].strip())
        except:
            count = 0
        bp = bdb.Breakpoint.bpbynumber[bpnum]
        if bp:
            bp.ignore = count
            if count > 0:
                reply = 'Will ignore next '
                if count > 1:
                    reply = reply + '%d crossings' % count
                else:
                    reply = reply + '1 crossing'
                print >> self.stdout, reply + ' of breakpoint %d.' % bpnum
            else:
                print >> self.stdout, 'Will stop next time breakpoint',
                print >> self.stdout, bpnum, 'is reached, unless it is disabled.'

    def do_clear(self, arg):
        if not self.VerifyNotPostMortem('clear'): return

        """Three possibilities, tried in this order:
        clear -> clear all breaks, ask for confirmation
        clear file:lineno -> clear all breaks at file:lineno
        clear bpno bpno ... -> clear breakpoints by number"""
        if not arg:
            try:
                self.stdout.write('Clear all breaks? ')
                reply = self.stdin.readline()
            except EOFError:
                reply = 'no'
            reply = reply.strip().lower()
            if reply in ('y', 'yes'):
                self.clear_all_breaks()
            return
        if ':' in arg:
            # Make sure it works for "clear C:\foo\bar.py:12"
            i = arg.rfind(':')
            filename = arg[:i]
            arg = arg[i+1:]
            try:
                lineno = int(arg)
            except:
                err = "Invalid line number (%s)" % arg
            else:
                err = self.clear_break(filename, lineno)
            if err: print >> self.stdout, '***', err
            return
        numberlist = arg.split()
        for i in numberlist:
            err = self.clear_bpbynumber(i)
            if err:
                print >> self.stdout, '***', err
            else:
                print >> self.stdout, 'Deleted breakpoint %s ' % (i,)
    do_cl = do_clear # 'c' is already an abbreviation for 'continue'

    def do_where(self, arg):
        self.print_stack_trace()
    do_w = do_where
    do_bt = do_where

    def do_up(self, arg):
        if self.curindex == 0:
            print >> self.stdout, '*** Oldest frame'
        else:
            self.curindex = self.curindex - 1
            self.curframe = self.stack[self.curindex][0]
            self.print_stack_entry(self.stack[self.curindex])
            self.lineno = None
    do_u = do_up

    def do_down(self, arg):
        if self.curindex + 1 == len(self.stack):
            print >> self.stdout, '*** Newest frame'
        else:
            self.curindex = self.curindex + 1
            self.curframe = self.stack[self.curindex][0]
            self.print_stack_entry(self.stack[self.curindex])
            self.lineno = None
    do_d = do_down

    def do_step(self, arg):
        if not self.VerifyNotPostMortem('step'): return

        self.set_step()
        return 1
    do_s = do_step

    def do_next(self, arg):
        if not self.VerifyNotPostMortem('next'): return

        self.set_next(self.curframe)
        return 1
    do_n = do_next

    def do_return(self, arg):
        if not self.VerifyNotPostMortem('return'): return

        self.set_return(self.curframe)
        return 1
    do_r = do_return

    def do_continue(self, arg):
        if not self.VerifyNotPostMortem('continue'): return

        self.set_continue()
        return 1
    do_c = do_cont = do_continue

    def do_jump(self, arg):
        if not self.VerifyNotPostMortem('jump'): return

        if self.curindex + 1 != len(self.stack):
            print >> self.stdout, "*** You can only jump within the bottom frame"
            return
        try:
            arg = int(arg)
        except ValueError:
            print >> self.stdout, "*** The 'jump' command requires a line number."
        else:
            try:
                # Do the jump, fix up our copy of the stack, and display the
                # new position
                self.curframe.f_lineno = arg
                self.stack[self.curindex] = self.stack[self.curindex][0], arg
                self.print_stack_entry(self.stack[self.curindex])
            except ValueError, e:
                print >> self.stdout, '*** Jump failed:', e
    do_j = do_jump

    def do_debug(self, arg):
        sys.settrace(None)
        globals = self.curframe.f_globals
        locals = self.curframe.f_locals
        p = Pdb(self.stdin, self.stdout)
        print >> self.stdout, "ENTERING RECURSIVE DEBUGGER"
        sys.call_tracing(p.run, (arg, globals, locals))
        print >> self.stdout, "LEAVING RECURSIVE DEBUGGER"
        sys.settrace(self.trace_dispatch)
        self.lastcmd = p.lastcmd

    def do_quit(self, arg):
        self.set_quit()
        return 1
    do_q = do_quit
    do_exit = do_quit

    def do_EOF(self, arg):
        print >> self.stdout, '',
        self.set_quit()
        return 1

    def do_args(self, arg):
        f = self.curframe
        co = f.f_code
        dict = f.f_locals
        n = co.co_argcount
        if not n:
            print >> self.stdout, "No arguments to the current function."
            return
        if co.co_flags & 4: n = n+1
        if co.co_flags & 8: n = n+1
        for i in range(n):
            name = co.co_varnames[i]
            print >> self.stdout, name, '=',
            if name in dict: print >> self.stdout, dict[name]
            else: print >> self.stdout, "*** undefined ***"
    do_a = do_args

    def do_retval(self, arg):
        if '__return__' in self.curframe.f_locals:
            print >> self.stdout, self.curframe.f_locals['__return__']
        else:
            print >> self.stdout, '*** Not yet returned!'
    do_rv = do_retval

    def _getval(self, arg):
        try:
            return eval(arg, self.curframe.f_globals,
                        self.curframe.f_locals)
        except:
            t, v = sys.exc_info()[:2]
            if isinstance(t, str):
                exc_type_name = t
            else: exc_type_name = t.__name__
            print >> self.stdout, '***', exc_type_name + ':', `v`
            raise

    def do_p(self, arg):
        try:
            print >> self.stdout, repr(self._getval(arg))
        except:
            pass

    def do_pp(self, arg):
        try:
            pprint.pprint(self._getval(arg), self.stdout)
        except:
            pass

    def do_list(self, arg):
        last = None
        if arg:
            try:
                x = eval(arg, {}, {})
                if type(x) == type(()):
                    first, last = x
                    first = int(first)
                    last = int(last)
                    if last < first:
                        # Assume it's a count
                        last = first + last
                else:
                    first = max(1, int(x) - 5)
            except:
                print >> self.stdout, '*** Error in argument:', `arg`
                return
        elif self.lineno is None:
            first = max(1, self.curframe.f_lineno - 5)
        else:
            first = self.lineno + 1
        if last is None:
            last = first + 10
        filename = self.curframe.f_code.co_filename
        print  >> self.stdout, '***', filename
        breaklist = self.get_file_breaks(filename)
        try:
            for lineno in range(first, last+1):
                line = linecache.getline(filename, lineno)
                if not line:
                    print >> self.stdout, '[EOF]'
                    break
                else:
                    s = `lineno`.rjust(3)
                    if len(s) < 4: s = s + ' '
                    if lineno in breaklist: s = s + 'B'
                    else: s = s + ' '
                    if lineno == self.curframe.f_lineno:
                        s = s + '->'
                    print >> self.stdout, s + '\t' + line,
                    self.lineno = lineno
        except KeyboardInterrupt:
            pass
    do_l = do_list

    def do_whatis(self, arg):
        try:
            value = eval(arg, self.curframe.f_globals,
                            self.curframe.f_locals)
        except:
            t, v = sys.exc_info()[:2]
            if type(t) == type(''):
                exc_type_name = t
            else: exc_type_name = t.__name__
            print >> self.stdout, '***', exc_type_name + ':', `v`
            return
        code = None
        # Is it a function?
        try: code = value.func_code
        except: pass
        if code:
            print >> self.stdout, 'Function', code.co_name
            return
        # Is it an instance method?
        try: code = value.im_func.func_code
        except: pass
        if code:
            print >> self.stdout, 'Method', code.co_name
            return
        # None of the above...
        print >> self.stdout, type(value)

    def do_alias(self, arg):
        args = arg.split()
        if len(args) == 0:
            keys = self.aliases.keys()
            if len(keys) == 0:
                print >> self.stdout, 'No aliases set.'
                return
            keys.sort()
            for alias in keys:
                print >> self.stdout, "%s = %s" % (alias, self.aliases[alias])
            return
        if len(args) == 1:
            if args[0] in self.aliases:
                print >> self.stdout, "%s = %s" % (args[0], self.aliases[args[0]])
            else:
                print >> self.stdout, "No such alias '" + args[0] + "'."
        else:
            self.aliases[args[0]] = ' '.join(args[1:])
            print >> self.stdout, "%s = %s" % (args[0], self.aliases[args[0]])

    def do_unalias(self, arg):
        args = arg.split()
        if len(args) == 0: return
        if args[0] in self.aliases:
            del self.aliases[args[0]]
            print >> self.stdout, "deleted alias '" + args[0] + "'."
        else:
            print >> self.stdout, "No such alias '" + args[0] + "'."

    # Print a traceback starting at the top stack frame.
    # The most recently entered frame is printed last;
    # this is different from dbx and gdb, but consistent with
    # the Python interpreter's stack trace.
    # It is also consistent with the up/down commands (which are
    # compatible with dbx and gdb: up moves towards 'main()'
    # and down moves towards the most recent stack frame).

    def print_stack_trace(self):
        try:
            for frame_lineno in self.stack:
                self.print_stack_entry(frame_lineno)
        except KeyboardInterrupt:
            pass

    def print_stack_entry(self, frame_lineno, prompt_prefix=line_prefix):
        frame, lineno = frame_lineno
        if frame is self.curframe:
            print >> self.stdout, '>',
        else:
            print >> self.stdout, ' ',
        print >> self.stdout, self.format_stack_entry(frame_lineno, prompt_prefix)

    def lookupmodule(self, filename):
        """Helper function for break/clear parsing -- may be overridden."""
        root, ext = os.path.splitext(filename)
        if ext == '':
            filename = filename + '.py'
        if os.path.isabs(filename):
            return filename
        for dirname in sys.path:
            while os.path.islink(dirname):
                dirname = os.readlink(dirname)
            fullname = os.path.join(dirname, filename)
            if os.path.exists(fullname):
                return fullname
        return None

    #
    # help_* method were moved to server console.
    #



def run(statement, globals=None, locals=None):
    Pdb().run(statement, globals, locals)

def runeval(expression, globals=None, locals=None):
    return Pdb(sys.stdin, sys.stdout).runeval(expression, globals, locals)

def runctx(statement, globals, locals):
    # B/W compatibility
    run(statement, globals, locals)

def runcall(*args):
    return Pdb(sys.stdin, sys.stdout).runcall(*args)

def set_active():
    global g_fActive
    g_fActive = True

def set_trace():
    if not g_fActive:
        return
    Pdb().set_trace()

# Post-Mortem interface

def post_mortem(t, stdin = sys.stdin, stdout = sys.stdout):
    p = Pdb(stdin, stdout, fPostMortem = True)
    p.reset()
    while t.tb_next is not None:
        t = t.tb_next
    p.interaction(t.tb_frame, t)

def pm():
    post_mortem(sys.last_traceback)


# Main program for testing

TESTCMD = 'import x; x.main()'

def test():
    run(TESTCMD)

# print help
def help():
    for dirname in sys.path:
        fullname = os.path.join(dirname, 'pdb.doc')
        if os.path.exists(fullname):
            sts = os.system('${PAGER-more} '+fullname)
            if sts: print '*** Pager exit status:', sts
            break
    else:
        print 'Sorry, can\'t find the help file "pdb.doc"',
        print 'along the Python search path'

# When invoked as main program, invoke the debugger on a script
if __name__=='__main__':
    #
    # Import self so when debugee references this module it will see the
    # correct values of the g_host and g_port globals.
    #
    import rpdb
    ret = rpdb.main()
    sys.exit(ret)

